@@ -13,27 +13,43 @@ __status__ = "Development"
 
 class MeasurementFunctionUtils:
     def __init__(
-        self, xvariables, ydims, str_repeat_corr_dims, verbose, templ, use_err_corr_dict
+        self,
+        xvariables,
+        uncxvariables,
+        ydims,
+        str_repeat_dims,
+        str_repeat_noncorr_dims,
+        verbose,
+        templ,
+        use_err_corr_dict,
+        broadcast_correlation,
+        param_fixed
     ):
         """
         Initialise MeasurementFunctionUtils
 
         :param xvariables: list of input quantity names, in same order as arguments in measurement function and with same exact names as provided in input datasets.
         :type xvariables: list(str)
-        :param str_repeat_corr_dims: list of dimension names to be used as repeated dims
-        :type str_repeat_corr_dims: list(str)
+        :param uncxvariables: list of input quantity names for which uncertainties should be propagated. Should be a subset of input quantity names. Defaults to None, in which case uncertainties on all input quantities are used.
+        :type uncxvariables: list(str), optional
+        :param str_repeat_noncorr_dims: list of dimension names to be used as repeated dims
+        :type str_repeat_noncorr_dims: list(str)
         :param verbose: boolean to set verbosity
         :type verbose: bool
         :param templ: templater object
         :type templ: punpy.digital_effects_table_template
         """
         self.xvariables = xvariables
+        self.uncxvariables = uncxvariables
         self.verbose = verbose
         self.ydims = ydims
-        self.str_repeat_corr_dims = str_repeat_corr_dims
+        self.str_repeat_noncorr_dims = str_repeat_noncorr_dims
+        self.str_repeat_dims = str_repeat_dims
         self.templ = templ
         self.repeat_dims_form = "structured"
         self.use_err_corr_dict = use_err_corr_dict
+        self.broadcast_correlation = broadcast_correlation
+        self.param_fixed = param_fixed
 
     def find_repeat_dim_corr(self, form, *args, store_unc_percent=False, ydims=None):
         """
@@ -49,8 +65,8 @@ class MeasurementFunctionUtils:
         :rtype: dict
         """
         repeat_dims_errcorrs = {}
-        for repeat_dim in self.ydims:
-            if repeat_dim in self.str_repeat_corr_dims:
+        for repeat_dim in self.ydims[0]:
+            if repeat_dim in self.str_repeat_noncorr_dims:
                 repeat_dims_errcorrs[repeat_dim] = {
                     "dim": repeat_dim,
                     "form": None,
@@ -135,7 +151,8 @@ class MeasurementFunctionUtils:
                 pass
             else:
                 raise ValueError(
-                    "punpy.measurement_function: Not all included uncertainty contributions have the same error correlation along the repeat_dims. Either don't use repeat_dims or use a different method, where components are propagated seperately."
+                    "punpy.measurement_function: Not all included uncertainty contributions have the same error correlation along the repeat_dims. Either don't use repeat_dims or use a different method, where components are propagated seperately. (%s,%s)"
+                    % (repeat_dims_errcorr["form"], uvar)
                 )
         else:
             if (
@@ -155,7 +172,7 @@ class MeasurementFunctionUtils:
                 pass
             else:
                 raise ValueError(
-                    "punpy.measurement_function: Not all included uncertainty contributions have the same error correlation along the repeat_dims. Either don't use repeat_dims or use a different method, where components are propagated seperately."
+                    "punpy.measurement_function: Not all included uncertainty contributions have the same error correlation along the %s dim. Either don't use repeat_dims and/or corr_dims or use a different method, where components are propagated seperately."%(repeat_dims_errcorr['dim'])
                 )
 
     def set_repeat_dims_form(self, repeat_dims_errcorrs):
@@ -168,13 +185,13 @@ class MeasurementFunctionUtils:
         :rtype: None
         """
         try:
-            if len(self.str_repeat_corr_dims) == 0 or len(repeat_dims_errcorrs) == 0:
+            if len(self.str_repeat_noncorr_dims) == 0 or len(repeat_dims_errcorrs) == 0:
                 self.repeat_dims_form = "None"
 
             elif np.all(
                 [
                     repeat_dims_errcorrs[repeat_dim]["form"] == "random"
-                    for repeat_dim in self.str_repeat_corr_dims
+                    for repeat_dim in self.str_repeat_noncorr_dims
                 ]
             ):
                 self.repeat_dims_form = "random"
@@ -182,7 +199,7 @@ class MeasurementFunctionUtils:
             elif np.all(
                 [
                     repeat_dims_errcorrs[repeat_dim]["form"] == "systematic"
-                    for repeat_dim in self.str_repeat_corr_dims
+                    for repeat_dim in self.str_repeat_noncorr_dims
                 ]
             ):
                 self.repeat_dims_form = "systematic"
@@ -193,7 +210,7 @@ class MeasurementFunctionUtils:
             if np.all(
                 [
                     repeat_dims_errcorrs[0][repeat_dim]["form"] == "random"
-                    for repeat_dim in self.str_repeat_corr_dims
+                    for repeat_dim in self.str_repeat_noncorr_dims
                 ]
             ):
                 self.repeat_dims_form = "random"
@@ -201,7 +218,7 @@ class MeasurementFunctionUtils:
             elif np.all(
                 [
                     repeat_dims_errcorrs[0][repeat_dim]["form"] == "systematic"
-                    for repeat_dim in self.str_repeat_corr_dims
+                    for repeat_dim in self.str_repeat_noncorr_dims
                 ]
             ):
                 self.repeat_dims_form = "systematic"
@@ -235,7 +252,7 @@ class MeasurementFunctionUtils:
             comps = dataset.unc[var].random_comps
             if comps is not None:
                 comps = list(comps.variables.mapping.keys())
-            if len(dataset[var].dims) < len(ydims):
+            if len(dataset[var].dims) < len(ydims[0]):
                 comps = []
         elif form == "sys" or form == "systematic":
             comps = dataset.unc[var].systematic_comps
@@ -247,14 +264,14 @@ class MeasurementFunctionUtils:
                 comps = list(comps.variables.mapping.keys())
             else:
                 comps = []
-            if len(dataset[var].dims) < len(ydims):
+            if len(dataset[var].dims) < len(ydims[0]):
                 comps_rand = dataset.unc[var].random_comps
                 if comps_rand is not None:
                     comps.append(*list(comps_rand.variables.mapping.keys()))
 
         else:
             compname = self.templ.make_ucomp_name(
-                form, store_unc_percent=store_unc_percent, var=var
+                var, form, store_unc_percent=store_unc_percent
             )
             if compname in comps:
                 comps = [compname]
@@ -283,9 +300,9 @@ class MeasurementFunctionUtils:
         if expand:
             if sizes_dict is None:
                 raise ValueError("sizes_dict should be set when using expand.")
-            if ydims is None:
+            if ydims[0] is None:
                 raise ValueError("ydims should be set when using expand.")
-            datashape = [sizes_dict[dim] for dim in ydims]
+            datashape = [sizes_dict[dim] for dim in ydims[0]]
 
         inputs = np.empty(len(self.xvariables), dtype=object)
         for iv, var in enumerate(self.xvariables):
@@ -296,21 +313,21 @@ class MeasurementFunctionUtils:
                         inputs[iv] = dataset[var].values
                         found = True
                         if expand:
-                            if inputs[iv].shape != datashape:
+                            if (inputs[iv].shape != datashape) and (not self.param_fixed[iv]):
                                 add_dims = [
-                                    dim for dim in ydims if dim not in dataset[var].dims
+                                    dim for dim in ydims[0] if dim not in dataset[var].dims
                                 ]
                                 for dim in add_dims:
-                                    tileshape = np.ones(len(ydims), dtype=int)
+                                    tileshape = np.ones(len(ydims[0]), dtype=int)
                                     if len(inputs[iv].shape) != len(datashape):
                                         tileshape[0] = sizes_dict[dim]
                                         inputs[iv] = np.tile(inputs[iv], tileshape)
                                         for idim2, dim2 in enumerate(add_dims):
                                             inputs[iv] = np.moveaxis(
-                                                inputs[iv], idim2, ydims.index(dim2)
+                                                inputs[iv], idim2, ydims[0].index(dim2)
                                             )
                                     else:
-                                        tileshape[ydims.index(dim)] = sizes_dict[dim]
+                                        tileshape[ydim[0].index(dim)] = sizes_dict[dim]
                                         inputs[iv] = np.tile(inputs[iv], tileshape)
                 elif var in dataset.keys():
                     inputs[iv] = dataset[var]
@@ -341,31 +358,34 @@ class MeasurementFunctionUtils:
         inputs_unc = np.empty(len(self.xvariables), dtype=object)
         for iv, var in enumerate(self.xvariables):
             inputs_unc[iv] = None
-            for dataset in args[0]:
-                if hasattr(dataset, "variables"):
-                    if var in dataset.variables:
-                        if ydims is not None:
-                            if np.all([dim in dataset[var].dims for dim in ydims]):
-                                inputs_unc[iv] = self.calculate_unc(form, dataset, var)
+            if var in self.uncxvariables:
+                for dataset in args[0]:
+                    if hasattr(dataset, "variables"):
+                        if var in dataset.variables:
+                            if ydims[0] is not None:
+                                if np.all([dim in dataset[var].dims for dim in ydims[0]]):
+                                    inputs_unc[iv] = self.calculate_unc(
+                                        form, dataset, var
+                                    )
 
-                            else:
-                                inputs_unc[iv] = self.calculate_unc_missingdim(
-                                    form,
-                                    dataset,
-                                    var,
-                                    expand=expand,
-                                    sizes_dict=sizes_dict,
-                                    ydims=ydims,
-                                )
+                                else:
+                                    inputs_unc[iv] = self.calculate_unc_missingdim(
+                                        form,
+                                        dataset,
+                                        var,
+                                        expand=expand,
+                                        sizes_dict=sizes_dict,
+                                        ydims=ydims,
+                                    )
 
-            if np.count_nonzero(inputs_unc[iv]) == 0:
-                inputs_unc[iv] = None
-            if inputs_unc[iv] is None:
-                if self.verbose:
-                    print(
-                        "%s uncertainty for variable %s not found in provided datasets. Zero uncertainty assumed."
-                        % (form, var)
-                    )
+                if np.count_nonzero(inputs_unc[iv]) == 0:
+                    inputs_unc[iv] = None
+                if inputs_unc[iv] is None:
+                    if self.verbose:
+                        print(
+                            "%s uncertainty for variable %s not found in provided datasets. Zero uncertainty assumed."
+                            % (form, var)
+                        )
         return inputs_unc
 
     def calculate_unc(self, form, ds, var):
@@ -450,9 +470,9 @@ class MeasurementFunctionUtils:
         if expand:
             if sizes_dict is None:
                 raise ValueError("sizes_dict should be set when using expand.")
-            if ydims is None:
+            if ydims[0] is None:
                 raise ValueError("ydims should be set when using expand.")
-            datashape = [sizes_dict[dim] for dim in ydims]
+            datashape = [sizes_dict[dim] for dim in ydims[0]]
 
         if (form == "rand") and (self.repeat_dims_form != "random"):
             out = None
@@ -476,14 +496,14 @@ class MeasurementFunctionUtils:
             out = self.calculate_unc(form, ds, var)
 
         if expand and (out is not None):
-            add_dims = [dim for dim in ydims if dim not in ds[var].dims]
+            add_dims = [dim for dim in ydims[0] if dim not in ds[var].dims]
             for dim in add_dims:
-                tileshape = np.ones(len(ydims), dtype=int)
+                tileshape = np.ones(len(ydims[0]), dtype=int)
                 if len(out.shape) != len(datashape):
                     tileshape[0] = sizes_dict[dim]
-                    out = np.moveaxis(np.tile(out, tileshape), 0, ydims.index(dim))
+                    out = np.moveaxis(np.tile(out, tileshape), 0, ydims[0].index(dim))
                 else:
-                    tileshape[ydims.index(dim)] = sizes_dict[dim]
+                    tileshape[ydims[0].index(dim)] = sizes_dict[dim]
                     out = np.tile(out, tileshape)
         return out
 
@@ -507,26 +527,34 @@ class MeasurementFunctionUtils:
         inputs_corr = np.empty(len(self.xvariables), dtype=object)
         for iv, var in enumerate(self.xvariables):
             inputs_corr[iv] = None
-            for dataset in args[0]:
-                if hasattr(dataset, "variables"):
-                    if var in dataset.variables:
-                        if len(dataset[var].dims) < len(ydims):
-                            inputs_corr[iv] = self.calculate_corr_missingdim(
-                                form,
-                                dataset,
-                                var,
-                                expand=expand,
-                                sizes_dict=sizes_dict,
-                                ydims=ydims,
-                            )
-                        else:
-                            inputs_corr[iv] = self.calculate_corr(form, dataset, var)
-            if inputs_corr[iv] is None:
-                if self.verbose:
-                    print(
-                        "%s error-correlation for variable %s not found in provided datasets."
-                        % (form, var)
-                    )
+            if var in self.uncxvariables:
+                for dataset in args[0]:
+                    if hasattr(dataset, "variables"):
+                        if var in dataset.variables:
+                            if len(dataset[var].dims) < len(ydims):
+                                inputs_corr[iv] = self.calculate_corr_missingdim(
+                                    form,
+                                    dataset,
+                                    var,
+                                    expand=expand,
+                                    sizes_dict=sizes_dict,
+                                    ydims=ydims,
+                                )
+                            else:
+                                inputs_corr[iv] = self.calculate_corr(
+                                    form, dataset, var
+                                )
+
+                # check if corr is filled with zeros
+                if not np.any(inputs_corr[iv]):
+                    inputs_corr[iv] = None
+
+                if inputs_corr[iv] is None:
+                    if self.verbose:
+                        print(
+                            "%s error-correlation for variable %s not found in provided datasets."
+                            % (form, var)
+                        )
 
         return inputs_corr
 
@@ -546,17 +574,24 @@ class MeasurementFunctionUtils:
         sli = list([slice(None)] * ds[var].ndim)
         var_dims = ds[var].dims
         for i in range(len(sli)):
-            if var_dims[i] in self.str_repeat_corr_dims:
+            if var_dims[i] in self.str_repeat_dims:
                 sli[i] = 0
         dsu = ds.unc[var][tuple(sli)]
         if form == "tot":
             if self.use_err_corr_dict:
-                return [unc.err_corr_dict_numdim() for unc in dsu]
+                return [
+                    self.convert_err_corr_dict_to_num(unc.err_corr_dict(), var_dims)
+                    for unc in dsu
+                ]
             else:
                 return dsu.total_err_corr_matrix().values
         elif form == "stru":
             if self.use_err_corr_dict:
-                return [unc.err_corr_dict_numdim() for unc in dsu if unc.is_structured]
+                return [
+                    self.convert_err_corr_dict_to_num(unc.err_corr_dict(), var_dims)
+                    for unc in dsu
+                    if unc.is_structured
+                ]
             else:
                 return dsu.structured_err_corr_matrix().values
         elif form == "rand":
@@ -581,7 +616,9 @@ class MeasurementFunctionUtils:
             try:
                 uvar = "%s_%s" % (form, var)
                 if self.use_err_corr_dict:
-                    return dsu[uvar].err_corr_dict_numdim()
+                    return self.convert_err_corr_dict_to_num(
+                        dsu[uvar].err_corr_dict(), var_dims
+                    )
                 else:
                     return dsu[uvar].err_corr_matrix().values
             except:
@@ -592,12 +629,31 @@ class MeasurementFunctionUtils:
                 uvar = keys[uvar_ids]
                 if len(uvar) > 0:
                     if self.use_err_corr_dict:
-                        return dsu[uvar[0]].err_corr_dict_numdim()
+                        return self.convert_err_corr_dict_to_num(
+                            dsu[uvar[0]].err_corr_dict(), var_dims
+                        )
                     else:
                         return dsu[uvar[0]].err_corr_matrix().values
                 else:
                     return None
 
+    def convert_err_corr_dict_to_num(self, err_corr_dict, var_dims):
+        """
+        Function to convert err_corr dictionary with dimension names as key, to dictionary with dimension indices as keys
+
+        :param err_corr_dict: err_corr dictionary with dimension names as key
+        :type err_corr_dict: dict
+        :param var_dims: list of variable dimensions, in correct order
+        :type var_dims: list
+        :return: err_corr dictionary with dimension indices as key
+        :rtype: dict
+        """
+        err_corr_dict_numdim = {}
+        for idim, dim in enumerate(var_dims):
+            if dim in err_corr_dict.keys():
+                err_corr_dict_numdim[str(idim)] = err_corr_dict[dim]
+        return err_corr_dict_numdim
+
     def calculate_corr_missingdim(
         self, form, ds, var, ydims=None, sizes_dict=None, expand=False
     ):
@@ -621,19 +677,22 @@ class MeasurementFunctionUtils:
         :rtype: np.ndarray
         """
         sli = [
-            slice(None) if (ydim not in self.str_repeat_corr_dims) else 0
+            slice(None) if (ydim not in self.str_repeat_noncorr_dims) else 0
             for ydim in ds[var].dims
         ]
         dsu = ds.unc[var][tuple(sli)]
         vardims = [
-            ydim for ydim in ds[var].dims if (ydim not in self.str_repeat_corr_dims)
+            ydim for ydim in ds[var].dims if (ydim not in self.str_repeat_noncorr_dims)
         ]
 
-        outdims = [ydim for ydim in ydims if ydim not in self.str_repeat_corr_dims]
+        outdims = [ydim for ydim in ydims if ydim not in self.str_repeat_noncorr_dims]
         missingdims = [
             ydim
             for ydim in ydims
-            if ((ydim not in ds[var].dims) and (ydim not in self.str_repeat_corr_dims))
+            if (
+                (ydim not in ds[var].dims)
+                and (ydim not in self.str_repeat_noncorr_dims)
+            )
         ]
 
         missingshape = [sizes_dict[dim] for dim in missingdims]
@@ -659,8 +718,16 @@ class MeasurementFunctionUtils:
 
         if expand and (out is not None):
             out_1 = cm.expand_errcorr_dims(out, vardims, outdims, sizes_dict)
+            if self.broadcast_correlation=="syst" or self.broadcast_correlation=="systematic":
+                broadcast_correlation=np.ones((missinglen, missinglen))
+            elif self.broadcast_correlation=="rand" or self.broadcast_correlation=="random":
+                broadcast_correlation=np.eye(missinglen)
+            elif isinstance(self.broadcast_correlation, np.ndarray):
+                broadcast_correlation=self.broadcast_correlation
+            else:
+                raise ValueError("punpy.measurement_function_utils: The provided broadcast_correlation is not valid.")
             out_2 = cm.expand_errcorr_dims(
-                np.ones((missinglen, missinglen)), missingdims, outdims, sizes_dict
+                broadcast_correlation, missingdims, outdims, sizes_dict
             )
             out = out_1.dot(out_2)
 
